#!/bin/bash

set -e

basetheme="prototype"
newtheme="STARTER_KIT"
target_dir="."
basetheme_dir=$(pwd)
newtheme_dir="$target_dir/$newtheme"

main() {
  configure $@
  confirm
  make_subtheme
}

configure() {
  local newtheme_ovrrd=$1
  local target_dir_ovrrd=$2
  newtheme=${newtheme_ovrrd:=$newtheme}
  target_dir=${target_dir_ovrrd:=$target_dir}
  newtheme_dir="$target_dir/$newtheme"
}

confirm() {
  info "Generating a new theme named %s in the directory %s\n" $newtheme $target_dir
  read -p "Continue? [Y/n]: " confirmed
  if [[ ! $confirmed == "Y" ]]; then
    printf "Cancelled.\n"
  else
    printf "Generating new subtheme in %s/%s...\n" $target_dir $newtheme
    sleep 1
  fi
}


make_subtheme() {
  mkdir -p $target_dir
  copy $basetheme_dir $newtheme_dir
  cd $newtheme_dir
  list_files_like $basetheme\.\* | bulk_rename "prototype" "$newtheme"
  cleanup
  theme_text > $newtheme.theme
  info_text > $newtheme.info.yml

  info "Updating %s category to %s...\n" $newtheme.layouts.yml $newtheme
  sed -i "s%category: Prototype%category: $newtheme%g" $newtheme.layouts.yml

  info "Updating template file components to %s...\n" "@$newtheme-components"
  find ./templates -type f | xargs sed -i "s%@components%@$newtheme-components%g"

  info "Done!\n"
}

cleanup() {
  remove=( \
    "./.git" \
    "./build" \
    "./templates/ui_patterns" \
  )

  for file in ${remove[@]}; do
    remove $file
  done
}

list_files_like() {
  find . -name "$1"
}

copy() {
	info "Copying %s to %s...\n" $1 $2
  rm -rf /tmp/$newtheme
  cp -r $1 /tmp/$newtheme
  mv /tmp/$newtheme $2
}

remove() {
	info "Removing %s\n" $1
  rm -rf $1
}

# Filenames read from stdin, replaces pattern (arg 1) in filename w/ replace (arg 2).
bulk_rename() {
	info "Bulk renaming files matching \"%s\" with \"%s\"...\n" "$1" "$2"
  sed "p;s%$1%$2%g" | xargs -n2 mv
}

# Replace pattern (arg 1) w/ replace (arg 2) in filename (arg 3)
replace() {
	info "Replacing occurences of \"%s\" with \"%s\" in %s...\n" "$1" "$2" "$3"
  sed -i s%$1%$2%g $3
}

theme_text() {
	info "Generating %s.theme...\n" $newtheme
  cat <<-EOD
	<?php

	/**
	* @file
	* Functions to support theming.
	*/
	EOD
}

info_text() {
	info "Generating %s.info.yml...\n" $newtheme
  cat <<-EOD
	name: $newtheme
	type: theme
	description: Prototype sub-theme
	package: $newtheme
	core: 8.x
	base theme: prototype
	libraries:
	  - $newtheme/global
	libraries-override:
	  prototype/global: $newtheme/global
	  prototype/search_form: $newtheme/search_form

	regions:
	  header: Header
	  primary_menu: Primary Menu
	  secondary_menu: Secondary Menu
	  highlighted: Highlighted
	  help: Help
	  content: Content
	  sidebar_first: First Sidebar
	  sidebar_second: Second Sidebar
	  footer: Footer
	  breadcrumb: Breadcrumb

	component-libraries:
	  $newtheme-components:
	    paths:
	      - components
	      - components/components
	      - components/navigation
        - components/regions
    $newtheme-icons:
      paths:
        - images/icons
	EOD
}

info() {
	(>&2 printf "$1" ${@:2})
}

main $@
